@@ -37,6 +37,40 @@ def to_array(commandline):
     return merged_split
 
 
+def is_human_friendly(command):
+    # type: (text_type) -> bool
+    """
+    AKA "Does this command contain any capital letters?"
+    """
+    for char in command:
+        if char.isupper():
+            return True
+    return False
+
+
+def get_app_name_prefix(commandline):
+    # type: (text_type) -> text_type
+    """
+    On macOS, get which app this command is part of.
+    """
+    command_with_path = to_array(commandline)[0]
+    command = os.path.basename(command_with_path)
+    for part in command_with_path.split("/"):
+        if '.' not in part:
+            continue
+
+        name, suffix = part.rsplit(".", 1)
+        if suffix not in ["app", "framework"]:
+            continue
+
+        if name == command:
+            continue
+
+        return name + "/"
+
+    return ""
+
+
 def get_command(commandline):
     # type: (text_type) -> text_type
     """
@@ -68,6 +102,7 @@ def get_command(commandline):
             '-a',
             '-d',
             '--debug',
+            '--disable',
             '-l',
             '-n',
             '-p',
@@ -81,11 +116,22 @@ def get_command(commandline):
             '-W2'
         ])
 
-    if command in ["bash", "sh", "perl", "node"]:
+    if command == "sudo":
+        return get_sudo_command(commandline)
+
+    if command == "node":
+        return get_generic_script_command(commandline, ["--max_old_space_size"])
+
+    if command in ["bash", "sh", "perl"]:
         return get_generic_script_command(commandline)
 
+    app_name_prefix = get_app_name_prefix(commandline)
+    if is_human_friendly(command):
+        # Already human friendly, prefer keeping it short
+        app_name_prefix = ""
+
     if len(command) < 25:
-        return command
+        return app_name_prefix + command
 
     command_split = command.split(".")
     if len(command_split) > 1:
@@ -98,7 +144,7 @@ def get_command(commandline):
             # next to last part
             command = command_split[-2]
 
-    return command
+    return app_name_prefix + command
 
 
 def get_python_command(commandline):
@@ -141,6 +187,19 @@ def get_python_command(commandline):
     return python
 
 
+def get_sudo_command(commandline):
+    # type: (text_type) -> text_type
+    without_sudo = commandline[5:].strip()
+    if not without_sudo:
+        return "sudo"
+
+    if without_sudo.startswith('-'):
+        # Give up on options
+        return "sudo"
+
+    return "sudo " + get_command(without_sudo)
+
+
 def prettify_fully_qualified_java_class(class_name):
     # type: (text_type) -> text_type
     split = class_name.split('.')
@@ -218,7 +277,7 @@ def get_generic_script_command(commandline, ignore_switches=[]):
     # type: (text_type, List[text_type]) -> text_type
     array = to_array(commandline)
 
-    while len(array) > 1 and array[1] in ignore_switches:
+    while len(array) > 1 and array[1].split("=")[0] in ignore_switches:
         del array[1]
 
     vm = os.path.basename(array[0])
