@@ -1,5 +1,7 @@
 # coding=utf-8
 
+import os
+
 from px import px_commandline
 
 
@@ -148,6 +150,45 @@ def test_get_command_resque():
     assert px_commandline.get_command("resqued-0.7.12 a b c") == "resqued-0.7.12"
 
 
+def test_get_command_sudo():
+    assert px_commandline.get_command("sudo") == "sudo"
+    assert px_commandline.get_command("sudo python /usr/bin/hej gris --flaska") == "sudo hej"
+
+    # We could support flags at some point, but until proven necessary we'll
+    # fall back on just "sudo" when we get flags.
+    assert px_commandline.get_command("sudo -B python /usr/bin/hej") == "sudo"
+
+
+def test_get_command_sudo_with_space_in_command_name(tmpdir):
+    # Create a file name with a space in it
+    spaced_path = tmpdir.join("i contain spaces")
+    spaced_path.write_binary(b"")
+    spaced_name = str(spaced_path)
+
+    # Verify splitting of the spaced file name
+    assert px_commandline.get_command("sudo " + spaced_name) == "sudo i contain spaces"
+
+    # Verify splitting with more parameters on the line
+    assert px_commandline.get_command("sudo " + spaced_name + " parameter") == \
+        "sudo i contain spaces"
+
+
+def test_get_command_sudo_with_space_in_path(tmpdir):
+    # Create a file name with a space in it
+    spaced_dir = tmpdir.join("i contain spaces")
+    os.mkdir(str(spaced_dir))
+    spaced_path = spaced_dir + "/runme"
+
+    spaced_path.write_binary(b"")
+    spaced_name = str(spaced_path)
+
+    # Verify splitting of the spaced file name
+    assert px_commandline.get_command("sudo " + spaced_name) == "sudo runme"
+
+    # Verify splitting with more parameters on the line
+    assert px_commandline.get_command("sudo " + spaced_name + " parameter") == "sudo runme"
+
+
 def test_get_command_interpreters():
     assert px_commandline.get_command("ruby") == "ruby"
     assert px_commandline.get_command("ruby /some/path/apa.rb") == "apa.rb"
@@ -173,3 +214,75 @@ def test_get_command_unicode():
 def test_get_command_ruby_switches():
     assert px_commandline.get_command(
         "/usr/bin/ruby -W0 /usr/local/bin/brew.rb install rust") == "brew.rb"
+
+
+def test_get_homebrew_commandline():
+    # Source: https://github.com/walles/px/issues/72
+    assert px_commandline.get_command(" ".join([
+        "/usr/local/Homebrew/Library/Homebrew/vendor/portable-ruby/current/bin/ruby",
+        "-W0",
+        "--disable=gems,did_you_mean,rubyopt",
+        "/usr/local/Homebrew/Library/Homebrew/brew.rb",
+        "upgrade"
+    ])) == "brew.rb"
+
+
+def test_node_max_old_space():
+    assert px_commandline.get_command(
+        "node --max_old_space_size=4096 scripts/start.js") == "start.js"
+
+
+def test_macos_app():
+    assert px_commandline.get_command("/".join([
+        "/System",
+        "Library",
+        "CoreServices",
+        "Dock.app",
+        "Contents",
+        "XPCServices",
+        "com.apple.dock.external.extra.xpc",
+        "Contents",
+        "MacOS",
+        "com.apple.dock.external.extra"
+    ])) == "Dock/extra"
+
+    # https://github.com/walles/px/issues/73
+    assert px_commandline.get_command("/".join([
+        "/Applications",
+        "Firefox.app",
+        "Contents",
+        "MacOS",
+        "plugin-container.app",
+        "Contents",
+        "MacOS",
+        "plugin-container"
+    ])) == "Firefox/plugin-container"
+
+    # Note that if we have spaces inside of the path, the path in
+    # question needs to be valid on the local system for it to
+    # be deciphered properly by px. So this path is from an actual
+    # path, but with all spaces removed.
+    assert px_commandline.get_command("/".join([
+        "/Applications",
+        "VisualStudioCode.app",
+        "Contents",
+        "Frameworks",
+        "CodeHelper(Renderer).app",
+        "Contents",
+        "MacOS",
+        "CodeHelper(Renderer)"
+    ])) == "CodeHelper(Renderer)"
+
+    assert px_commandline.get_command("/Applications/iTerm.app/Contents/MacOS/iTerm2") == "iTerm2"
+
+    # Don't duplicate the .app name
+    assert px_commandline.get_command("/".join([
+        "/System",
+        "Library",
+        "PrivateFrameworks",
+        "IDS.framework",
+        "identityservicesd.app",
+        "Contents",
+        "MacOS",
+        "identityservicesd"
+    ])) == "IDS/identityservicesd"
